@@ -14,21 +14,20 @@ def train_model(model):
     # Sanity check on the Bayesian Network
     assert isinstance(model, BayesNet), "'model' has to be a BayesNet class"
 
-
     ####################
     ## Start training ##
     ####################
 
-    print ("\n")
-    print ("#"*38)
-    print ("## Training the model with seed %d ##" % (model.options['seed']))
-    print ("#"*38)
-    print ("\n")
+    print("\n")
+    print("#" * 38)
+    print("## Training the model with seed %d ##" % (model.options["seed"]))
+    print("#" * 38)
+    print("\n")
 
     model.iterate()
 
     print("\n")
-    print("#"*23)
+    print("#" * 23)
     print("## Training finished ##")
-    print("#"*23)
+    print("#" * 23)
     print("\n")
