@@ -21,44 +21,49 @@ import numpy as np
 
 # General class for probability distributions
 class Distribution(object):
-    """ General class for a statistical distribution """
+    """General class for a statistical distribution"""
+
     def __init__(self, dim):
         self.dim = dim
 
     def density(self):
-        """ General method to calculate density """
+        """General method to calculate density"""
         pass
+
     def loglik(self):
-        """ General method to calculate log likelihood """
+        """General method to calculate log likelihood"""
         pass
+
     def sample(self):
-        """ General method to sample from the distribution """
+        """General method to sample from the distribution"""
         pass
+
     def entropy(self):
-        """ General method to calculate entropy """
+        """General method to calculate entropy"""
         pass
+
     def updateExpectations(self):
-        """ General method to update expectations """
+        """General method to update expectations"""
         pass
 
     def getParameters(self):
-        """ General getter function for parameters """
+        """General getter function for parameters"""
         return self.params
 
-    def setParameters(self,**params):
-        """ General setter function for parameters """
+    def setParameters(self, **params):
+        """General setter function for parameters"""
         self.params = params
 
     def getExpectation(self):
-        """ General getter function for expectations """
-        return self.expectations['E']
+        """General getter function for expectations"""
+        return self.expectations["E"]
 
     def getExpectations(self):
-        """ General setter function for expectations """
+        """General setter function for expectations"""
         return self.expectations
 
     def CheckDimensionalities(self):
-        """ General method to do a sanity check on the dimensionalities """
+        """General method to do a sanity check on the dimensionalities"""
         # p_dim = set(map(s.shape, self.params.values()))
         e_dim = set(map(s.shape, self.expectations.values()))
         # assert len(p_dim) == 1, "Parameters have different dimensionalities"
@@ -66,13 +71,14 @@ class Distribution(object):
         # assert e_dim == p_dim, "Parameters and Expectations have different dimensionality"
 
     def to_float32(self):
-        """ Convert numpy arrays from float64 to float32 """
-        for i in self.params.keys(): self.params[i] = self.params[i].astype(np.float32)
-        for i in self.expectations.keys(): self.expectations[i] = self.expectations[i].astype(np.float32)
-
+        """Convert numpy arrays from float64 to float32"""
+        for i in self.params.keys():
+            self.params[i] = self.params[i].astype(np.float32)
+        for i in self.expectations.keys():
+            self.expectations[i] = self.expectations[i].astype(np.float32)
 
     def removeDimensions(self, axis, idx):
-        """ General method to remove undesired dimensions
+        """General method to remove undesired dimensions
 
         PARAMETERS
         ----------
@@ -82,13 +88,15 @@ class Distribution(object):
             indices of the elements to remove
         """
         assert axis <= len(self.dim)
-        assert s.all(idx < self.dim[axis])
-        for k in self.params.keys(): self.params[k] = s.delete(self.params[k], idx, axis)
-        for k in self.expectations.keys(): self.expectations[k] = s.delete(self.expectations[k], idx, axis)
-        self.updateDim(axis=axis, new_dim=self.dim[axis]-len(idx))
+        assert np.all(idx < self.dim[axis])
+        for k in self.params.keys():
+            self.params[k] = np.delete(self.params[k], idx, axis)
+        for k in self.expectations.keys():
+            self.expectations[k] = np.delete(self.expectations[k], idx, axis)
+        self.updateDim(axis=axis, new_dim=self.dim[axis] - len(idx))
 
     def updateDim(self, axis, new_dim):
-        """ Method to update the dimensionality of a particular axis. 
+        """Method to update the dimensionality of a particular axis.
         This method is a bit inefficient but we store dimensionalities with tuples and they cannot be modified
 
         PARAMETERS
