@@ -16,7 +16,10 @@ import scipy as s
 import numpy.ma as ma
 import numpy as np
 
-from .variational_nodes import Unobserved_Variational_Node, Unobserved_Variational_Mixed_Node
+from .variational_nodes import (
+    Unobserved_Variational_Node,
+    Unobserved_Variational_Mixed_Node,
+)
 from .basic_nodes import *
 from .Y_nodes import Y_Node
 from .Tau_nodes import TauD_Node
@@ -29,8 +32,10 @@ from mofapy2.core.utils import sigmoid, lambdafn
 ## General pseudodata nodes ##
 ##############################
 
+
 class PseudoY(Unobserved_Variational_Node):
-    """ General class for pseudodata nodes """
+    """General class for pseudodata nodes"""
+
     def __init__(self, dim, obs, groups, params=None, E=None):
         """
         PARAMETERS
@@ -57,13 +62,13 @@ class PseudoY(Unobserved_Variational_Node):
             self.params = {}
 
         # Create a boolean mask of the data to handle missing values
-        self.mask = ma.getmask( ma.masked_invalid(self.obs) )
-        self.obs[np.isnan(self.obs)] = 0.
+        self.mask = ma.getmask(ma.masked_invalid(self.obs))
+        self.obs[np.isnan(self.obs)] = 0.0
 
         # Initialise expectation
         if E is not None:
             assert E.shape == dim, "Problems with the dimensionalities"
-            E[self.mask] = 0.
+            E[self.mask] = 0.0
         self.E = E
 
     def updateParameters(self, ix=None, ro=None):
@@ -71,7 +76,7 @@ class PseudoY(Unobserved_Variational_Node):
 
     def getMask(self, full=True):
         # Currently always full as stochastic inference not implemented for Gaussian nodes
-        #TODO: change when implementing SVI for non-Gaussian
+        # TODO: change when implementing SVI for non-Gaussian
         return self.mask
 
     def precompute(self, options=None):
@@ -79,14 +84,16 @@ class PseudoY(Unobserved_Variational_Node):
         pass
 
     def updateExpectations(self):
-        print("Error: expectation updates for pseudodata node depend on the type of likelihood. They have to be specified in a new class.")
+        print(
+            "Error: expectation updates for pseudodata node depend on the type of likelihood. They have to be specified in a new class."
+        )
         exit()
 
     def getExpectation(self, expand=True):
         return self.E
 
     def define_mini_batch(self, ix):
-        """ Method to define a mini-batch (only for stochastic inference) """
+        """Method to define a mini-batch (only for stochastic inference)"""
         # self.mini_batch = self.E[ix,:]
         pass
 
@@ -95,7 +102,7 @@ class PseudoY(Unobserved_Variational_Node):
         return self.getExpectation(expand)
 
     def getExpectations(self, expand=True):
-        return { 'E':self.getExpectation(expand) }
+        return {"E": self.getExpectation(expand)}
 
     # def getObservations(self):
     #     return self.obs
@@ -110,12 +117,15 @@ class PseudoY(Unobserved_Variational_Node):
         print("Not implemented")
         exit()
 
+
 ##################
 ## Seeger nodes ##
 ##################
 
+
 class PseudoY_Seeger(PseudoY):
-    """ General class for pseudodata nodes using the seeger approach """
+    """General class for pseudodata nodes using the seeger approach"""
+
     def __init__(self, dim, obs, groups, params=None, E=None):
         # Inputs:
         #  dim (2d tuple): dimensionality of each view
@@ -127,11 +137,12 @@ class PseudoY_Seeger(PseudoY):
         Z = self.markov_blanket["Z"].getExpectation()
         W = self.markov_blanket["W"].getExpectation()
         # self.params["zeta"] = np.dot(Z,W.T)
-        self.params["zeta"] = gpu_utils.dot( gpu_utils.array(Z),gpu_utils.array(W).T )
+        self.params["zeta"] = gpu_utils.dot(gpu_utils.array(Z), gpu_utils.array(W).T)
+
 
 class Tau_Seeger(Constant_Node):
-    """
-    """
+    """ """
+
     def __init__(self, dim, value):
         Constant_Node.__init__(self, dim=dim, value=value)
 
@@ -139,7 +150,7 @@ class Tau_Seeger(Constant_Node):
         return self.value
 
     def define_mini_batch(self, ix):
-        """ Method to define a mini-batch (only for stochastic inference) """
+        """Method to define a mini-batch (only for stochastic inference)"""
         # self.mini_batch = self.value[ix,:]
         pass
 
@@ -151,7 +162,8 @@ class Tau_Seeger(Constant_Node):
         return self.getValue()
 
     def getExpectations(self, expand=True):
-        return { 'E':self.getExpectation(expand) }
+        return {"E": self.getExpectation(expand)}
+
 
 class Poisson_PseudoY(PseudoY_Seeger):
     """
@@ -174,14 +186,19 @@ class Poisson_PseudoY(PseudoY_Seeger):
     The bound degrades with the presence of entries with large y_ij, so one should consider
     clipping overly large counts
     """
+
     def __init__(self, dim, obs, groups, params=None, E=None):
         # - dim (2d tuple): dimensionality of each view
         # - obs (ndarray): observed data
         # - E (ndarray): initial expected value of pseudodata
-        PseudoY_Seeger.__init__(self, dim=dim, obs=obs, groups=groups, params=params, E=E)
+        PseudoY_Seeger.__init__(
+            self, dim=dim, obs=obs, groups=groups, params=params, E=E
+        )
 
         # Initialise the observed data
-        assert np.all(s.mod(self.obs, 1) == 0), "Data must not contain float numbers, only integers"
+        assert np.all(
+            s.mod(self.obs, 1) == 0
+        ), "Data must not contain float numbers, only integers"
         assert np.all(self.obs >= 0), "Data must not contain negative numbers"
 
     def precompute(self, options):
@@ -190,7 +207,7 @@ class Poisson_PseudoY(PseudoY_Seeger):
 
     def ratefn(self, X):
         # Poisson rate function
-        return np.log(1+np.exp(X)) + 0.0001
+        return np.log(1 + np.exp(X)) + 0.0001
 
     def clip(self, threshold):
         # The local bound degrades with the presence of large values in the observed data, which should be clipped
@@ -199,15 +216,20 @@ class Poisson_PseudoY(PseudoY_Seeger):
     def updateExpectations(self):
         # Update the pseudodata
         tau = self.markov_blanket["Tau"].getValue()
-        self.E = self.params["zeta"] - sigmoid(self.params["zeta"])*(1-self.obs/self.ratefn(self.params["zeta"])) / tau
-        self.E[self.mask] = 0.
+        self.E = (
+            self.params["zeta"]
+            - sigmoid(self.params["zeta"])
+            * (1 - self.obs / self.ratefn(self.params["zeta"]))
+            / tau
+        )
+        self.E[self.mask] = 0.0
 
         # regress out feature-wise mean from the pseudodata
         self.means = self.E.mean(axis=0).data
         self.E -= self.means
 
     def calculateELBO(self):
-        """ Compute Evidence Lower Bound """
+        """Compute Evidence Lower Bound"""
 
         Wtmp = self.markov_blanket["W"].getExpectations()
         Ztmp = self.markov_blanket["Z"].getExpectations()
@@ -219,22 +241,23 @@ class Poisson_PseudoY(PseudoY_Seeger):
 
         # Precompute terms
         ZW = Z.dot(W.T)
-        ZZWW = np.square(ZW) - np.dot(np.square(Z),np.square(W).T) + ZZ.dot(WW.T)
+        ZZWW = np.square(ZW) - np.dot(np.square(Z), np.square(W).T) + ZZ.dot(WW.T)
 
         # term1 = 0.5*tau*(ZW - zeta)**2
-        term1 = 0.5*tau*(ZZWW - 2*ZW*zeta + np.square(zeta))
-        term2 = (ZW - zeta)*(sigmoid(zeta)*(1.-self.obs/self.ratefn(zeta)))
-        term3 = self.ratefn(zeta) - self.obs*np.log(self.ratefn(zeta))
+        term1 = 0.5 * tau * (ZZWW - 2 * ZW * zeta + np.square(zeta))
+        term2 = (ZW - zeta) * (sigmoid(zeta) * (1.0 - self.obs / self.ratefn(zeta)))
+        term3 = self.ratefn(zeta) - self.obs * np.log(self.ratefn(zeta))
 
         elbo = -(term1 + term2 + term3)
-        elbo[mask] = 0.
+        elbo[mask] = 0.0
 
         # I AM NOT SURE WHY NAs are generated...
         np.isnan(elbo).sum()
-        elbo[np.isnan(elbo)] = 0.
-        
+        elbo[np.isnan(elbo)] = 0.0
+
         return elbo.sum()
 
+
 class Bernoulli_PseudoY(PseudoY_Seeger):
     """
     Class for a Bernoulli (0,1 data) pseudodata node
@@ -249,18 +272,21 @@ class Bernoulli_PseudoY(PseudoY_Seeger):
         yhat_ij = zeta_ij - f'(zeta_ij)/tau
                 = zeta_ij - 4*(sigmoid(zeta_ij) - y_ij)
     """
+
     def __init__(self, dim, obs, groups, params=None, E=None):
         # - dim (2d tuple): dimensionality of each view
         # - obs (ndarray): observed data
         # - E (ndarray): initial expected value of pseudodata
-        PseudoY_Seeger.__init__(self, dim=dim, obs=obs, groups=groups, params=params, E=E)
+        PseudoY_Seeger.__init__(
+            self, dim=dim, obs=obs, groups=groups, params=params, E=E
+        )
 
         # Initialise the observed data
-        assert np.all( (self.obs==0) | (self.obs==1) ), "Data must be binary"
+        assert np.all((self.obs == 0) | (self.obs == 1)), "Data must be binary"
 
     def updateExpectations(self):
         # Update the pseudodata
-        self.E = self.params["zeta"] - 4.*(sigmoid(self.params["zeta"]) - self.obs)
+        self.E = self.params["zeta"] - 4.0 * (sigmoid(self.params["zeta"]) - self.obs)
 
         # regress out feature-wise mean from the pseudodata
         self.means = self.E.mean(axis=0).data
@@ -272,10 +298,10 @@ class Bernoulli_PseudoY(PseudoY_Seeger):
         W = self.markov_blanket["W"].getExpectation()
         mask = self.getMask()
 
-        tmp = gpu_utils.asnumpy( gpu_utils.dot( gpu_utils.array(Z),gpu_utils.array(W).T ) )
+        tmp = gpu_utils.asnumpy(gpu_utils.dot(gpu_utils.array(Z), gpu_utils.array(W).T))
 
-        lb = self.obs*tmp - np.log(1.+np.exp(tmp))
-        lb[mask] = 0.
+        lb = self.obs * tmp - np.log(1.0 + np.exp(tmp))
+        lb[mask] = 0.0
 
         return lb.sum()
 
@@ -284,31 +310,33 @@ class Bernoulli_PseudoY(PseudoY_Seeger):
 ## Jaakkola nodes ##
 ####################
 
+
 class Tau_Jaakkola(Node):
     """
     Local Parameter that needs to be optimised in the Jaakkola approach.
     For more details see Supplementary Methods
     """
+
     def __init__(self, dim, value):
         Node.__init__(self, dim=dim)
 
-        if isinstance(value,(int,float)):
+        if isinstance(value, (int, float)):
             self.value = value * np.ones(dim)
         else:
             assert value.shape == dim, "Dimensionality mismatch"
             self.value = value
 
     def define_mini_batch(self, ix):
-        """ Method to define a mini-batch (only for stochastic inference) """
+        """Method to define a mini-batch (only for stochastic inference)"""
         # self.mini_batch = self.E[ix,:]
         pass
 
     # TODO change: define a proper mini-batch !!
     def get_mini_batch(self, expand=True):
         return self.getExpectation(expand)
-        
+
     def updateExpectations(self):
-        self.value = 2*lambdafn(self.markov_blanket["Y"].getParameters()["zeta"])
+        self.value = 2 * lambdafn(self.markov_blanket["Y"].getParameters()["zeta"])
 
     def getValue(self):
         return self.value
@@ -317,10 +345,12 @@ class Tau_Jaakkola(Node):
         return self.getValue()
 
     def getExpectations(self, expand=True):
-        return { 'E':self.getValue(), 'lnE':np.log(self.getValue()) }
+        return {"E": self.getValue(), "lnE": np.log(self.getValue())}
 
     def removeFactors(self, idx, axis=None):
         pass
+
+
 class Bernoulli_PseudoY_Jaakkola(PseudoY):
     """
     Class for a Bernoulli pseudodata node using the Jaakkola approach:
@@ -339,18 +369,19 @@ class Bernoulli_PseudoY_Jaakkola(PseudoY):
     NOTE: For this class to work the noise variance tau needs to be updated according to
         tau_ij <- 2*lambadfn(xi_ij)
     """
+
     def __init__(self, dim, obs, groups, params=None, E=None):
         PseudoY.__init__(self, dim=dim, obs=obs, groups=groups, params=params, E=E)
 
         # Initialise the observed data
-        assert np.all( (self.obs==0) | (self.obs==1) ), "Data must be binary"
+        assert np.all((self.obs == 0) | (self.obs == 1)), "Data must be binary"
 
     def precompute(self, options):
         self.updateParameters(ix=None, ro=None)
         self.updateExpectations()
 
     def updateExpectations(self):
-        self.E = (2.*self.obs - 1.)/(4.*lambdafn(self.params["zeta"]))
+        self.E = (2.0 * self.obs - 1.0) / (4.0 * lambdafn(self.params["zeta"]))
 
         # regress out feature-wise mean from the pseudodata
         self.means = self.E.mean(axis=0).data
@@ -359,7 +390,11 @@ class Bernoulli_PseudoY_Jaakkola(PseudoY):
     def updateParameters(self, ix=None, ro=None):
         Z = self.markov_blanket["Z"].getExpectations()
         W = self.markov_blanket["W"].getExpectations()
-        self.params["zeta"] = np.sqrt(np.square(Z["E"].dot(W["E"].T)) - np.dot(np.square(Z["E"]), np.square(W["E"].T)) + np.dot(Z["E2"],W["E2"].T))
+        self.params["zeta"] = np.sqrt(
+            np.square(Z["E"].dot(W["E"].T))
+            - np.dot(np.square(Z["E"]), np.square(W["E"].T))
+            + np.dot(Z["E2"], W["E2"].T)
+        )
 
     def calculateELBO(self):
         # Compute Evidence Lower Bound using the lower bound to the likelihood
@@ -373,27 +408,30 @@ class Bernoulli_PseudoY_Jaakkola(PseudoY):
 
         # calculate E(Z)E(W)
         ZW = Z.dot(SW.T)
-        ZW[mask] = 0.
+        ZW[mask] = 0.0
 
         # Calculate E[(ZW_nd)^2]
         # this is equal to E[\sum_{k != k} z_k w_k z_k' w_k'] + E[\sum_{k} z_k^2 w_k^2]
-        tmp1 = np.square(ZW) - np.dot(np.square(Z),np.square(SW).T) # this is for terms in k != k'
-        tmp2 = ZZ.dot(SWW.T) # this is for terms in k = k'
+        tmp1 = np.square(ZW) - np.dot(
+            np.square(Z), np.square(SW).T
+        )  # this is for terms in k != k'
+        tmp2 = ZZ.dot(SWW.T)  # this is for terms in k = k'
         EZZWW = tmp1 + tmp2
 
         # calculate elbo terms
-        term1 = 0.5 * ((2.*self.obs - 1.)*ZW - zeta)
-        term2 = - np.log(1 + np.exp(-zeta))
-        term3 = - 1/(4 * zeta) *  np.tanh(zeta/2.) * (EZZWW - zeta**2)
+        term1 = 0.5 * ((2.0 * self.obs - 1.0) * ZW - zeta)
+        term2 = -np.log(1 + np.exp(-zeta))
+        term3 = -1 / (4 * zeta) * np.tanh(zeta / 2.0) * (EZZWW - zeta**2)
 
         lb = term1 + term2 + term3
-        lb[mask] = 0.
+        lb[mask] = 0.0
 
         return lb.sum()
 
-#-------------------------------------------------------------------------------
+
+# -------------------------------------------------------------------------------
 # Zero inflated data: mixed node implementation
-#-------------------------------------------------------------------------------
+# -------------------------------------------------------------------------------
 # TODO in the data processing make sure that the data is centered around the zeros
 # TODO create initialiser
 # TODO build Markov Blanket in each sub-node. for the tau we need to give the right Y
@@ -408,24 +446,27 @@ class Zero_Inflated_PseudoY_Jaakkola(Unobserved_Variational_Mixed_Node):
     Appropriate wiring is done in the markov blanket so that tau jaakola sees the
     jaakola Y and normal tau sees the normal Y
     """
+
     def __init__(self, dim, obs, params=None, E=None):
         self.dim = dim
-        
+
         self.all_obs = obs
         if type(self.all_obs) != ma.MaskedArray:
             self.all_obs = ma.masked_invalid(self.all_obs)
 
         # identify the zeros, nonzeros and nas and store their positions in masks
-        self.zeros = (self.all_obs == 0)
-        self.nonzeros = ~self.zeros # this masks includes the nas
+        self.zeros = self.all_obs == 0
+        self.nonzeros = ~self.zeros  # this masks includes the nas
         self.nas = ma.getmask(self.all_obs)
-        self.mask = self.nas   # TODO mask to be used when calculating variance explained 
+        self.mask = self.nas  # TODO mask to be used when calculating variance explained
 
-        self.sparsity = self.zeros.sum()/(self.zeros.sum() + self.nonzeros.sum())
-        print('using zero inflated noise model with sparsity ', self.sparsity)
+        self.sparsity = self.zeros.sum() / (self.zeros.sum() + self.nonzeros.sum())
+        print("using zero inflated noise model with sparsity ", self.sparsity)
 
         # initialise the jaakola node with nas and non zeros masked
-        obs_jaakola = obs.copy()  # TODO if obs is already a masked array should we update mask ?
+        obs_jaakola = (
+            obs.copy()
+        )  # TODO if obs is already a masked array should we update mask ?
         obs_jaakola[self.nonzeros] = np.nan
         # instead:?
         # self.notnas = np.logical_not(self.nas)
@@ -434,24 +475,28 @@ class Zero_Inflated_PseudoY_Jaakkola(Unobserved_Variational_Mixed_Node):
 
         # Initialise a y node where the zeros and nas are masked
         obs_normal = obs.copy()
-        obs_normal[self.zeros]= np.nan  # nas are already masked so no need to
+        obs_normal[self.zeros] = np.nan  # nas are already masked so no need to
         self.normal_node = Y_Node(dim, obs_normal)
 
         self.nodes = [self.normal_node, self.jaakola_node]
 
-
     def addMarkovBlanket(self, **kwargs):
         self.jaakola_node.addMarkovBlanket(**kwargs)
 
         # NOTE here we make sure that the non-zero Y node sees the corresponding Tau
-        if not hasattr(self.normal_node, 'markov_blanket'):
+        if not hasattr(self.normal_node, "markov_blanket"):
             self.normal_node.markov_blanket = {}
 
-        for k,v in kwargs.items():
+        for k, v in kwargs.items():
             if k in self.normal_node.markov_blanket.keys():
-                print("Error: " + str(k) + " is already in the markov blanket of " + str(self.normal_node))
+                print(
+                    "Error: "
+                    + str(k)
+                    + " is already in the markov blanket of "
+                    + str(self.normal_node)
+                )
                 exit(1)
-            elif k == 'Tau':
+            elif k == "Tau":
                 self.normal_node.markov_blanket[k] = v.tau_normal
             else:
                 self.normal_node.markov_blanket[k] = v
@@ -467,8 +512,10 @@ class Zero_Inflated_PseudoY_Jaakkola(Unobserved_Variational_Mixed_Node):
 
     def getExpectation(self, expand=True):
         E = self.normal_node.getExpectation().copy()
-        pseudo_y = self.jaakola_node.getExpectation() # TODO Is this in any ways comparable to E?
-        E[self.zeros] = pseudo_y[self.zeros] 
+        pseudo_y = (
+            self.jaakola_node.getExpectation()
+        )  # TODO Is this in any ways comparable to E?
+        E[self.zeros] = pseudo_y[self.zeros]
         return E
 
     def get_mini_batch(self, expand=True):
@@ -499,38 +546,48 @@ class Zero_Inflated_Tau_Jaakkola(Unobserved_Variational_Mixed_Node):
         # all contained in the Zero_Inflated_Tau_Jaakkola node
         N = len(groups)
         self.tau_jaakola = Tau_Jaakkola((N, dim[1]), value)
-        self.tau_normal  = TauD_Node(dim, pa, pb, qa, qb, groups, qE)
+        self.tau_normal = TauD_Node(dim, pa, pb, qa, qb, groups, qE)
 
         self.nodes = [self.tau_jaakola, self.tau_normal]
 
     def addMarkovBlanket(self, **kwargs):
         # create a marov blanket for tau containing the zero inflated Y
-        if not hasattr(self, 'markov_blanket'):
+        if not hasattr(self, "markov_blanket"):
             self.markov_blanket = {}
 
         # create the markov blanket for the jaakola tau, wiring the corresonding Y
-        if not hasattr(self.tau_jaakola, 'markov_blanket'):
+        if not hasattr(self.tau_jaakola, "markov_blanket"):
             self.tau_jaakola.markov_blanket = {}
 
-        for k,v in kwargs.items():
+        for k, v in kwargs.items():
             if k in self.tau_jaakola.markov_blanket.keys():
-                print("Error: " + str(k) + " is already in the markov blanket of " + str(self.tau_jaakola))
+                print(
+                    "Error: "
+                    + str(k)
+                    + " is already in the markov blanket of "
+                    + str(self.tau_jaakola)
+                )
                 exit(1)
-            elif k == 'Y':
+            elif k == "Y":
                 self.tau_jaakola.markov_blanket[k] = v.jaakola_node
-                self.markov_blanket[k] = v # put the 'mixed' Y node in the mb of self
+                self.markov_blanket[k] = v  # put the 'mixed' Y node in the mb of self
             else:
                 self.tau_jaakola.markov_blanket[k] = v
 
         # create the markov blanket for the normal tau, wiring the corressponding Y
-        if not hasattr(self.tau_normal, 'markov_blanket'):
+        if not hasattr(self.tau_normal, "markov_blanket"):
             self.tau_normal.markov_blanket = {}
 
-        for k,v in kwargs.items():
+        for k, v in kwargs.items():
             if k in self.tau_normal.markov_blanket.keys():
-                print("Error: " + str(k) + " is already in the markov blanket of " + str(self.tau_normal))
+                print(
+                    "Error: "
+                    + str(k)
+                    + " is already in the markov blanket of "
+                    + str(self.tau_normal)
+                )
                 exit(1)
-            elif k == 'Y':
+            elif k == "Y":
                 self.tau_normal.markov_blanket[k] = v.normal_node
             else:
                 self.tau_normal.markov_blanket[k] = v
@@ -541,13 +598,13 @@ class Zero_Inflated_Tau_Jaakkola(Unobserved_Variational_Mixed_Node):
         tau_jk = self.tau_jaakola.getExpectations(expand=True)
 
         # Merge
-        zeros = self.markov_blanket['Y'].zeros
-        E['E'][zeros], E['lnE'][zeros] = tau_jk['E'][zeros], tau_jk['lnE'][zeros]
+        zeros = self.markov_blanket["Y"].zeros
+        E["E"][zeros], E["lnE"][zeros] = tau_jk["E"][zeros], tau_jk["lnE"][zeros]
 
         return E
 
     def getExpectation(self, expand=True):
-        return self.getExpectations(expand)['E']
+        return self.getExpectations(expand)["E"]
 
     def get_mini_batch(self, expand=True):
         return self.getExpectation(expand=True)
